@@ -23,7 +23,8 @@
 #' 
 #' @param x design matrix, matrix of size \eqn{N \times p}.
 #' @param y response matrix, matrix of size \eqn{N \times K}.
-#' @param intercept should the model include intercept parameters
+#' @param intercept should the model include intercept parameters.
+#' @param weights sample weights, vector of size \eqn{N \times K}.
 #' @param grouping grouping of features, a factor or vector of length \eqn{p}. Each element of the factor/vector specifying the group of the feature. 
 #' @param groupWeights the group weights, a vector of length \eqn{m} (the number of groups). 
 #' @param parameterWeights a matrix of size \eqn{K \times p}. 
@@ -46,7 +47,7 @@
 #'
 #' ## Simulate from Y=XB+E, the dimension of Y is N x K, X is N x p, B is p x K 
 #' 
-#' N <- 50 #number of samples
+#' N <- 100 #number of samples
 #' p <- 50 #number of features
 #' K <- 25  #number of groups
 #' 
@@ -55,21 +56,38 @@
 #' Y1 <-X1%*%B+matrix(rnorm(N*K,0,1),N,K)
 #' 
 #' ##Do cross validation
-#' lambda <- lsgl.lambda(X1, Y1, alpha = 1, d = 25L, lambda.min = 0.5, intercept = FALSE)
-#' fit.cv <- lsgl.cv(X1, Y1, alpha = 1, fold = 2, lambda = lambda, intercept = FALSE)
+#' lambda <- lsgl.lambda(X1, Y1, alpha = 1, d = 15L, lambda.min = 5, intercept = FALSE)
+#' fit.cv <- lsgl.cv(X1, Y1, alpha = 1, lambda = lambda, intercept = FALSE)
 #' 
 #' ## Cross validation errors (estimated expected generalization error)
 #' Err(fit.cv)
 #' @author Martin Vincent
-#' @useDynLib lsgl .registration=TRUE
+#' @useDynLib lsgl, .registration=TRUE
 #' @export
+#' @importFrom utils packageVersion
 lsgl.cv <- function(x, y, intercept = TRUE,
+		weights = NULL,
 		grouping = factor(1:ncol(x)), 
 		groupWeights = c(sqrt(ncol(y)*table(grouping))),
 		parameterWeights =  matrix(1, nrow = ncol(y), ncol = ncol(x)), 
 		alpha = 1, lambda, fold = 10L, cv.indices = list(), max.threads = 2L,
 		algorithm.config = lsgl.standard.config) 
 {
+	
+	if(!is.matrix(y)) {
+		y <- as.matrix(y)
+	}
+	
+	if(nrow(x) != nrow(y)) {
+		stop("x and y must have the same number of rows")
+	}
+	
+	if(!is.null(weights)) {
+		if(!all(dim(y) == dim(weights))) {
+			stop("w and y must have the same dimensions")
+		}
+	}
+	
 	# Get call
 	cl <- match.call()
 	
@@ -86,19 +104,51 @@ lsgl.cv <- function(x, y, intercept = TRUE,
 	
 	# create data
 	group.names <- if(is.null(colnames(y))) 1:ncol(y) else colnames(y)
-	data <- create.sgldata(x, y, group.names = group.names)
+	data <- create.sgldata(x, y, weights = weights, group.names = group.names)
+	
+	# Print info
+	if(algorithm.config$verbose) {
+		
+		n_fold <- if(length(cv.indices) == 0) fold else length(cv.indices)
+		
+		cat("\nRunning lsgl", n_fold, "fold cross validation ")
+		if(data$sparseX & data$sparseY) {
+			cat("(sparse design and response matrices)")
+		}
+		if(data$sparseX & !data$sparseY) {
+			cat("(sparse design matrix)")
+		}
+		if(!data$sparseX & data$sparseY) {
+			cat("(sparse response matrix)")
+		}
+		
+		cat("\n\n")
+		
+		print(data.frame('Samples: ' = print_with_metric_prefix(nrow(x)), 
+						'Features: ' = print_with_metric_prefix(data$n.covariate), 
+						'Models: ' = print_with_metric_prefix(ncol(y)), 
+						'Groups: ' = print_with_metric_prefix(length(unique(grouping))), 
+						'Parameters: ' = print_with_metric_prefix(length(parameterWeights)),
+						check.names = FALSE), 
+				row.names = FALSE, digits = 2, right = TRUE)
+		cat("\n")
+	}
+	
 	
 	# call SglOptimizer function
-	if(data$sparseX) {
-		
-		res <- sgl_cv("lsgl_sparse", "lsgl", data, grouping, groupWeights, parameterWeights, alpha, lambda, fold, cv.indices, max.threads, algorithm.config)
-		
+	if(!is.null(weights)) {
+		obj <- "lsgl_w_"
 	} else {
-		
-		res <- sgl_cv("lsgl_dense", "lsgl", data, grouping, groupWeights, parameterWeights, alpha, lambda, fold, cv.indices, max.threads, algorithm.config)
-		
+		obj <- "lsgl_"
 	}
 	
+	callsym <- paste(obj, if(data$sparseX) "xs_" else "xd_", if(data$sparseY) "ys" else "yd", sep = "")
+	
+	res <- sgl_cv(callsym, "lsgl", data, grouping, groupWeights, parameterWeights, alpha, lambda, fold, cv.indices, max.threads, algorithm.config)
+	
+	# Add weights
+	res$weights <- weights
+	
 	# Add true response
 	res$Y.true <- y
 	
